/*
   Class Name:

      COctree.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class is used to store a mesh in a octree.
*/


#ifndef _COCTREE_H_
#define _COCTREE_H_

#include<stdio.h>


enum NODE_ID
{
   TOP_FRONT_LEFT = 0,
   TOP_FRONT_RIGHT,
   TOP_BACK_LEFT,
   TOP_BACK_RIGHT,
   BOTTOM_FRONT_LEFT,
   BOTTOM_FRONT_RIGHT,
   BOTTOM_BACK_LEFT,
   BOTTOM_BACK_RIGHT
};


class COctree
{
   public:
      COctree();
      ~COctree();


      bool Initialize(float *vertexList, int numVerts, float *t1, float *t2,
                      int maxLevels, int maxTris);
      void Shutdown();

      bool isLeaf() { return m_isLeaf; }
      bool isVertices() { return (m_vertices != NULL); }

      float *GetVertices() { return m_vertices; }
      float *GetTexCoord1() { return m_texCoord1; }
      float *GetTexCoord2() { return m_texCoord2; }
      int GetNumTriangles() { return (int)(m_numPoints / 3); }
      float GetBoundingBoxSize() { return m_bbSize; }

      void GetBoundingBoxCenter(float &cx, float &cy, float &cz)
      {
         cx = m_bbCenter[0]; cy = m_bbCenter[1]; cz = m_bbCenter[2];
      }

      COctree **GetSubNodes() { return m_subNodes; }

   private:
      void CalculateBoundingBox(float *vertexList, int numVerts);
      void CreateNode(float *vertexList, int numVerts, float *t1, float *t2,
                      float *bbCenter, float bbSize);
      void CreateLeafNode(float *vList, int numVerts, float *t1, float *t2,
                          bool *bArray, int numTriangles, float *bbCenter,
                          float bbSize, int nodeId);
      float *GetNodeCenter(float *bbCenter, float bbSize, int nodeId);


   private:
      // Leaf means node has data, else its a node.
      bool m_isLeaf;

      // List of node's vertex points.
      float *m_vertices;
      int m_numPoints;

      // Lost of node's tex coords.
      float *m_texCoord1;
      float *m_texCoord2;

      // Node's bounding box data.
      float m_bbCenter[3];
      float m_bbSize;

      COctree *m_subNodes[8];
};

#endif


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com