/*
   Class Name:

      CSkyDome.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class represents a sky dome to be rendered over a 3d scene.
*/


#include"CSkyDome.h"

#define PI 3.141592654f    // Define for PI.
#define RADIANS PI / 180.0f// Turn degrees to radians.


CSkyDome::CSkyDome() : vertices(0), texCoords(0), numPoints(0)
{
}


CSkyDome::~CSkyDome()
{
   // Call shut down to make sure everything is cleared out.
   ShutDown();
}


bool CSkyDome::Initialize(float thetaAmt, float phiAmt, float radius)
{
   float vec[3] = {0};
   int index = 0;

	// Initialize the number of points necessary.
	numPoints = (int)((360 / thetaAmt) * (90 / phiAmt) * 4);

	// Allocate the points and tex coords list.
	vertices  = new float[numPoints * 3];
	texCoords = new float[numPoints * 2];

	// Loop through and calculate the points for the entire dome.
	for(int phi = 0; phi <= (90 - phiAmt); phi += (int)phiAmt)
	{
		for(int theta = 0; theta <= (360 - thetaAmt); theta += (int)thetaAmt)
		{
			// Compute the vertex at phi and theta.
			vertices[(index * 3) + 0] = radius * sinf(phi * RADIANS) * cosf(theta * RADIANS);
			vertices[(index * 3) + 1] = radius * sinf(phi * RADIANS) * sinf(theta * RADIANS);
			vertices[(index * 3) + 2] = radius * cosf(phi * RADIANS);

			// Normalize this vector.
         vec[0] = vertices[(index * 3) + 0];
         vec[1] = vertices[(index * 3) + 1];
         vec[2] = vertices[(index * 3) + 2];

         Normalize(vec);

			// Calculate this points texture coordinate.
         ComputeTextureCoord((index * 2), (index * 2) + 1, vec);
			index++;


			// Compute the vertex at phi + phiAmt and theta.
			vertices[(index * 3) + 0] = radius * sinf((phi + phiAmt) * RADIANS) *
                                     cosf(theta * RADIANS);
			vertices[(index * 3) + 1] = radius * sinf((phi + phiAmt) * RADIANS) *
                                     sinf(theta * RADIANS);
			vertices[(index * 3) + 2] = radius * cosf((phi + phiAmt) * RADIANS);
			
			// Create a normal out of this point.
         vec[0] = vertices[(index * 3) + 0];
         vec[1] = vertices[(index * 3) + 1];
         vec[2] = vertices[(index * 3) + 2];

         Normalize(vec);

			// Calculate this points texture coordinate.
         ComputeTextureCoord((index * 2), (index * 2) + 1, vec);	
			index++;


			// Compute the vertex at phi and theta + thetaAmt.
			vertices[(index * 3) + 0] = radius * sinf(RADIANS * phi) *
                                     cosf(RADIANS * (theta + thetaAmt));
			vertices[(index * 3) + 1] = radius * sinf(RADIANS * phi) *
                                     sinf(RADIANS * (theta + thetaAmt));
			vertices[(index * 3) + 2] = radius * cosf(RADIANS * phi);
			
			// Create a normal out of this point.
         vec[0] = vertices[(index * 3) + 0];
         vec[1] = vertices[(index * 3) + 1];
         vec[2] = vertices[(index * 3) + 2];

         Normalize(vec);

			// Calculate this points texture coordinate.
         ComputeTextureCoord((index * 2), (index * 2) + 1, vec);		
			index++;


			if(phi < 90)
			{
				// Compute the vertex at phi + phiAmt and theta + thetaAmt.
				vertices[(index * 3) + 0] = radius * sinf((phi + phiAmt) * RADIANS) *
                                        cosf(RADIANS * (theta + thetaAmt));
				vertices[(index * 3) + 1] = radius * sinf((phi + phiAmt) * RADIANS) *
                                        sinf(RADIANS * (theta + thetaAmt));
				vertices[(index * 3) + 2] = radius * cosf((phi + phiAmt) * RADIANS);
				
				// Create a normal out of this point.
            vec[0] = vertices[(index * 3) + 0];
            vec[1] = vertices[(index * 3) + 1];
            vec[2] = vertices[(index * 3) + 2];

            Normalize(vec);

				// Calculate this points texture coordinate.
            ComputeTextureCoord((index * 2), (index * 2) + 1, vec);
				index++;
			}
		}
	}

   return true;
}


void CSkyDome::Normalize(float vec[])
{
   // Normalize a vector.
   float length = (float)sqrt((vec[0] * vec[0]) + (vec[1] * vec[1]) +
                              (vec[2] * vec[2]));

   vec[0] /= length;
   vec[1] /= length;
   vec[2] /= length;
}


void CSkyDome::ComputeTextureCoord(int tu, int tv, float vec[])
{
   // Compute tex coord.
   texCoords[tu] = (float)(atan2(vec[0], vec[2]) / (PI * 2)) + 0.5f;
	texCoords[tv] = (float)(asinf(vec[1]) / PI) + 0.5f;
}


void CSkyDome::ShutDown()
{
   // Free all resources used by this sky dome.
   if(vertices != NULL)
      {
         delete[] vertices;
         vertices = NULL;
      }

   if(texCoords != NULL)
      {
         delete[] texCoords;
         texCoords = NULL;
      }

   numPoints = 0;
}


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com